function PD = Box(parameters, CreateDefaultFlag, app)

if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag
    Variable    ={'Voxelnx';'Voxelny';'Pitchx';'Pitchy';'zstep';'height';'alternatex';'alternatey';'pr'};                                  % name of each variable used for this scruture
    Description ={'Width x';'Width y';'Pitch x';'Pitch y'; 'Voxel height';'Total height';'Flag: Alternate in x';'Flag: Alternate in y';'Pressure'};                  % description of each variable
    Value       ={1;1;3;3;0.5;40;0;0;60};                                              % default values 
    Unit        ={'voxel';'voxel';'um';'um';'um';'um';'logical';'logical';'mbar'};             % units of each variable
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);  % build the default parameters table
    save([app.parameters_path], 'ParametersTable');                     % save the default parameters table to disk    
    PD = [];                                                            % return empty PD
else
    
    % Get the variable values from the parameters table:
    
    [nR,nC]=size(parameters);
    for k=1:nR
        temp=parameters{(parameters.Row{k}),'Value'};
        obj.(parameters.Row{k})=temp{:};
        temp=[];
    end
    
    PD=MakeBox(obj);       
end
end

function PD=MakeBox(obj)
% ------------------------
% |                      |
% |                      |
% |                      |
% |                      |
% |                      |
%-------------------------

xend    = (obj.Voxelnx-1) * obj.Pitchx;
yend    = (obj.Voxelny-1) * obj.Pitchy;

obj.X = []; obj.Y = []; obj.Z = [];

if rem(obj.Voxelny,2)==0
    evenrows=1;
else
    evenrows=0;
end

if obj.alternatex
    offsetx = 0.5;
else
    offsetx = 0;
end

if obj.alternatey
    offsety = 0.5;
else
    offsety = 0;
end

zcount = 0;

for i=obj.zstep:obj.zstep:obj.height  % z = height
    zcount = zcount + 1;
    xlayer = []; ylayer = []; zlayer = [];
    
    for j=1:1:obj.Voxelny            % y = rows
        
        for k=1:1:obj.Voxelnx        % x = colums
            
            if rem(zcount,2)==1 % odd z, start on top
                if rem(j,2)==1 % odd
                    next.x = (k-1)*obj.Pitchx;
                else
                    next.x = xend-(k-1)*obj.Pitchx;
                end
                next.y = (j-1-offsety)*obj.Pitchy;
                
            else % z is even, start on bottom and flip x direction
                
                if rem((j+evenrows),2)==1 % odd
                    next.x = xend-(k-1-offsetx)*obj.Pitchx;
                else
                    next.x = (k-1+offsetx)*obj.Pitchx;
                end
                next.y = yend-(j-1)*obj.Pitchy;
                
            end
            
            next.z = i;
            
            % insert the actual next point in the array
            xlayer = [xlayer; next.x];
            ylayer = [ylayer; next.y];
            zlayer = [zlayer; next.z];
            
        end
        
    end
    
    obj.X = [obj.X; xlayer];
    obj.Y = [obj.Y; ylayer];
    obj.Z = [obj.Z; zlayer];
    
end

PD.x=obj.X;
PD.y=obj.Y;
PD.z=obj.Z;
PD.pr = obj.pr*ones(length(PD.z),1);

end