function PD = CSV(parameters, CreateDefaultFlag, app)

if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag
    
    Variable    ={'CSVpressflag'; 'pr'};             % name of each variable used for this scruture
    Description ={'Use CSV pressure flag';'Pressure'};% description of each variable
    Value       ={0;50};                                           % default values 
    Unit        ={'logical';'mbar'};                          % units of each variable
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);  % build the default parameters table
    save([app.parameters_path], 'ParametersTable');                     % save the default parameters table to disk    
    PD = [];% return empty PD
else
        
    % Get the variable values from the parameters table:
    
    [nR,nC]=size(parameters);
    for k=1:nR
        temp=parameters{(parameters.Row{k}),'Value'};
        obj.(parameters.Row{k})=temp{:};
        temp=[];
    end
    
    if (length(parameters{'script','Value'}{:})==3) && (strcmp(parameters{'script','Value'}{:}{1},mfilename))
        %if a CSV was already loaded and we have its filename and path       
        obj.filename = parameters{'script','Value'}{:}{3};
        obj.pathname = parameters{'script','Value'}{:}{2};
    else
        % Load file from CSV
        [obj.filename, obj.pathname] =  uigetfile('*.csv', 'Select a comma separated values file', [app.parentfolder 'Printfiles\']);
    end
    if ~isequal(obj.filename,0)
        app.edStatus.Value = [obj.filename ' selected'];
        app.parameters{'script','Value'} ={{mfilename,obj.pathname,obj.filename}};
        PD=MakeCSV(obj); % call the function building the structure's voxels cloud
        %PD.script={'CSV',obj.pathname,obj.filename};
    else
        PD=[];
    end    
end
end
    
function PD=MakeCSV(obj)
    
        PD=[];
        PD = AD_PrintFile(fullfile(obj.pathname,obj.filename), PD);
        
        if ~obj.CSVpressflag % overwrite pressure with parameter value if flag is 0
            PD.pr = obj.pr*ones(length(PD.z),1);
        end
         if ~isfield(PD,'pr') % if no pressure column in the printfile
            PD.pr = obj.pr*ones(length(PD.z),1);
        end
end