function [PD] = CalibrationArray(parameters, CreateDefaultFlag, app)

if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag
    
    Variable    ={'vh';'ph';'pr';'pitchx';'pitchy';'numx'};  % name of each variable used for this scruture
    Description ={'Voxel height';'pillar height';'Pressure list';...
        'Pillar pitch in x'; 'Pillar pitch in y'; 'Number of pillars in x'};              % description of each variable
    Value       ={0.5;40;'161 160 150 140 130 120 110 100 90 80 70 60 50 40 30 20';...
        30;20;4};     % default values
    Unit        ={'um';'voxels';'mbar';'um'; 'um';'pillars'};                             % units of each variable
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);          % build the default parameters table with variable name as row name
    save([app.parameters_path], 'ParametersTable');                             % save the default parameters table to disk
    PD = [];                                                                    % return empty PD
else
    % Get the variable values from the parameters table:
    [nR,~]=size(parameters);
    for k=1:nR
        temp=parameters{(parameters.Row{k}),'Value'};                   % take the value for each variable
        if ischar(temp{:})                                              % convert from char to number array if needed
            temp2=str2num(temp{:});
            temp={temp2};
        end
        obj.(parameters.Row{k})=temp{:};                                % create the obj. structure based on the name of each variable
        temp=[];
    end
    PD=MakeCalibration(obj);                                            % call the function building the structure's voxels cloud
end
end

function PD=MakeCalibration(obj)

PD.x=[];
PD.y=[];
PD.z=[];
PD.pr=[];
N=length(obj.pr);
z=(1:obj.ph)'*obj.vh;
for i=1:1:N
    x = mod(i-1,obj.numx)*obj.pitchx;
    y = fix((N-i)/obj.numx)*obj.pitchy;
    xdat=ones(length(z),1).*x;
    ydat=ones(length(z),1).*y;
    pilpres=ones(length(z),1).*obj.pr(i);
    PD.x=[PD.x; xdat];
    PD.y=[PD.y; ydat];
    PD.z=[PD.z; z];
    PD.pr=[PD.pr; pilpres];
end
end