function [PD] = CalibrationPillar(parameters, CreateDefaultFlag, app)

if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag

    Variable    ={'vh';'n';'pr'};                                               % name of each variable used for this scruture
    Description ={'Voxel height';'section length';'Pressure list'};             % description of each variable
    Value       ={0.5;20;'160 150 140 130 120 110 100 90 80 70 60 50'};           % default values 
    Unit        ={'um';'voxels';'mbar'};                                        % units of each variable
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);          % build the default parameters table with variable name as row name
    save([app.parameters_path], 'ParametersTable');                             % save the default parameters table to disk    
    PD = [];                                                                    % return empty PD
else
    
    % Get the variable values from the parameters table:
    
    [nR,nC]=size(parameters); 
    for k=1:nR 
        temp=parameters{(parameters.Row{k}),'Value'};                   % take the value for each variable
        if ischar(temp{:})                                              %convert from char to number array if needed
            temp2=str2num(temp{:});
            temp={temp2};
        end
        obj.(parameters.Row{k})=temp{:};                                % create the obj. structure based on the name of each variable
        temp=[];
    end
    
    PD=MakeCalibration(obj);                                            % call the function building the structure's voxels cloud

end
end

function PD=MakeCalibration(obj)  

N=length(obj.pr);

z=[1:N*obj.n]'*obj.vh;
y=zeros(length(z),1);
x=y;

section=ones(obj.n,N).*obj.pr;

PD.x=x;
PD.y=y;
PD.z=z;
PD.pr=section(:);
end