function PD = Demo3D(parameters, CreateDefaultFlag, app)


if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag
    
    
    Variable    ={'r1';'r2';'h1';'h2';'h3';'h4';'d_pillar';'d_strand';'d_cube';'d_base';'strand_dr';'pr'};                                  % name of each variable used for this scruture
    Description ={'Cyl. base radius';'Top base radius';'Cyl. base Z level';'Top base Z level';'pillar and strands height';'prisma 1/2 height';'pillar voxel distance';'strand voxel distance';'base voxel distance';'prisma voxel distance';'strands radius reduction';'Pressure'};                  % description of each variable
    Value       ={30; 15; 10; 20; 140; 20; 1; 1; 1; 1; 3; 50};                                              % default values 
    Unit        ={'um';'um';'um';'um';'um';'um';'um';'um';'um';'um';'um';'mbar'};                                     % units of each variable
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);  % build the default parameters table
    save([app.parameters_path], 'ParametersTable');                     % save the default parameters table to disk    
    PD = [];                                                            % return empty PD
else
    
    % Get the variable values from the parameters table:
    
    [nR,nC]=size(parameters);
    for k=1:nR
        temp=parameters{(parameters.Row{k}),'Value'};
        obj.(parameters.Row{k})=temp{:};
        temp=[];
    end
    
    PD=MakeDemo3D(obj);      
end
end

function PD=MakeDemo3D(obj)

    obj.c=1;
    obj.strand_ro=(obj.r1+obj.r2)*0.5;
    obj.hs=obj.h3+obj.strand_ro*2^0.5;
    obj.strand_wo=linspace(0.25,2.25,5);
    obj.strand_wo=obj.strand_wo(1:end-1)*pi;
    obj.strand_zo=mean([obj.h1,obj.h2]);
    obj.strand_next=obj.strand_zo+obj.d_strand;
    
    obj.pillar_next=obj.h2+obj.d_pillar;
    obj.cube_d=1/(2^0.5)*obj.d_cube;
    obj.cube_next=obj.h3+obj.cube_d;
    v=[obj.r2,obj.h2]-[obj.r1,obj.h1];
    v=v/norm(v);
    obj.base_dz=obj.d_base*v(2);
    
    obj.top_d=obj.d_cube;
    
    obj.X=[];
    obj.Y=[];
    obj.Z=[];
    
    obj=demo(obj);
    
    PD.x=obj.X;
    PD.y=obj.Y;
    PD.z=obj.Z;
    PD.pr = obj.pr*ones(length(PD.z),1);
    
end

function obj=demo(obj)
obj.dz=obj.d_base;
for z=linspace(obj.dz,obj.h1,(obj.h1-obj.dz)/obj.dz+1)
    obj.z=z;
    obj=base(obj);
end
obj.z=obj.base_next;
while obj.z<obj.strand_zo
    obj=base(obj);
    obj.z=obj.base_next;
end
while obj.base_next<obj.h2
    [obj.z,index]=min([obj.strand_next,obj.base_next]);
    if index==1
        obj=strand(obj);
    else
        obj=base(obj);
    end
end
obj.z=obj.h2;
for r=linspace(obj.r2-obj.d_base,obj.d_base,(obj.r2-obj.d_base)/obj.d_base+1)
    obj.r=r;
    obj=base(obj);
end
while obj.pillar_next<=obj.h3
    [obj.z,index]=min([obj.pillar_next,obj.strand_next]);
    if index==1
        obj=pillar(obj);
    else
        obj=strand(obj);
    end
end
obj.c=1;
while obj.strand_next<obj.hs
    [obj.z,index]=min([obj.cube_next,obj.strand_next]);
    if index==1
        obj=cube(obj);
    else
        obj=strand(obj);
    end
end
while obj.cube_l>obj.h4*0.5
    obj=cube(obj);
    obj.z=obj.cube_next;
end
obj.top_l=obj.cube_l;
while obj.top_l>0
    obj=top(obj);
end
end

function obj=top(obj)
n=round(obj.top_l/obj.cube_d);
x=[linspace(0,obj.top_l-obj.top_d,n)';linspace(obj.top_l,obj.top_d-obj.top_l,2*n)';linspace(-obj.top_l,-obj.top_d,n)'];
y=[linspace(-obj.top_l,obj.top_l-obj.top_d,2*n)';linspace(obj.top_l,obj.top_d-obj.top_l,2*n)'];

if isempty(x)
    x=0;
    y=0;
end
obj.c=obj.c+1;
obj.X=[obj.X;x];
obj.Y=[obj.Y;y];
obj.Z=[obj.Z;ones(length(x),1)*obj.z];
obj.top_l=obj.top_l-obj.top_d;
end

function obj=cube(obj)

if obj.z<(obj.h3+obj.h4)
    obj.cube_z=obj.z-obj.h3;
    obj.cube_l=obj.cube_z;
else
    obj.cube_z=obj.z-obj.h4-obj.h3;
    obj.cube_l=obj.h4-obj.cube_z;
end

n=round(obj.cube_l/obj.cube_d);
x=[linspace(0,obj.cube_l-obj.cube_d,n)';linspace(obj.cube_l,obj.cube_d-obj.cube_l,2*n)';linspace(-obj.cube_l,-obj.cube_d,n)'];
y=[linspace(-obj.cube_l,obj.cube_l-obj.cube_d,2*n)';linspace(obj.cube_l,obj.cube_d-obj.cube_l,2*n)'];

if isempty(x)
    x=0;
    y=0;
end
obj.c=obj.c+1;
obj.X=[obj.X;x];
obj.Y=[obj.Y;y];
obj.Z=[obj.Z;ones(length(x),1)*obj.z];
obj.cube_next=obj.z+obj.cube_d;

end

function obj=pillar(obj)
obj.X=[obj.X;0];
obj.Y=[obj.Y;0];
obj.Z=[obj.Z;obj.z];
obj.pillar_next=obj.z+obj.d_pillar;
end

function obj=strand(obj)
obj.strand_z=obj.z-obj.strand_zo;
qr=2*pi/(obj.hs-obj.strand_zo)*obj.strand_z;
qw=pi/(obj.hs-obj.strand_zo)*obj.strand_z;
obj.strand_r=obj.strand_ro-obj.strand_dr*(1-cos(qr));
obj.strand_w=obj.strand_wo+pi*(1-cos(qw));

obj.X=[obj.X;obj.strand_r.*cos(obj.strand_w')];
obj.Y=[obj.Y;obj.strand_r.*sin(obj.strand_w)'];
obj.Z=[obj.Z;ones(length(obj.strand_w),1)*obj.z];
obj=strand_next(obj);
end

function obj=strand_next(obj)
dz=0.2;
qr=2*pi/(obj.hs-obj.strand_zo)*(obj.strand_z+dz);
qw=pi/(obj.hs-obj.strand_zo)*(obj.strand_z+dz);
obj.strand_n_r=obj.strand_ro-obj.strand_dr*(1-cos(qr));
obj.strand_n_w=obj.strand_wo(1)+pi*(1-cos(qw));
dx=obj.strand_n_r.*cos(obj.strand_n_w)-obj.strand_r.*cos(obj.strand_w(1));
dy=obj.strand_n_r.*sin(obj.strand_n_w)-obj.strand_r.*sin(obj.strand_w(1));
D=[dx,dy,dz];
D=D./norm(D);
obj.strand_next=obj.z+D(3)*obj.d_strand;
end

function obj=base(obj)
obj=base_r(obj);
obj=ring_w(obj);
obj.X=[obj.X;obj.r.*cos(obj.w)];
obj.Y=[obj.Y;obj.r.*sin(obj.w)];
obj.Z=[obj.Z;ones(length(obj.w),1)*obj.z];
obj.base_next=obj.z+obj.base_dz;
end

function obj=ring_w(obj)
dw=2*asin(obj.d_base/(2*obj.r));
obj.c=obj.c+1;
if mod(obj.c,2)==1
    obj.w=linspace(0,2*pi-dw,2*pi/dw)';
else
    obj.w=linspace(dw*0.5,2*pi-(dw*0.5),2*pi/dw)';
end
end

function obj=base_r(obj)
if obj.z<obj.h1
    obj.r=obj.r1;
end
if obj.z<obj.h2 && obj.z>=obj.h1
    obj.r=(obj.r2-obj.r1)/(obj.h2-obj.h1)*(obj.z-obj.h1)+obj.r1;
end
if obj.z==obj.h2
    obj.r=obj.r;
end
end
