function PD = DemoStructScript(parameters, CreateDefaultFlag, app)
% main function of the scrip, do not change except for variable,
% description, value and Unit cells
if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag
    %created the default parameter file in VCG/Standatd_structure_folder
    
%%%%%%%%%%%%%%% User-defined parameters appearing in the GUI %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Variable    ={'var1';'var2';'pr'};                                  % name of each variable used for this scruture
    Description ={'Variable 1';'Variable 2';'Pressure'};                % description of each variable
    Value       ={'1 0';0;60};                                          % default values 
    Unit        ={'1';'um';'mbar'};                                     % units of each variable
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);  % build the default parameters table
    save([app.parameters_path], 'ParametersTable');                     % save the default parameters table to disk    
    PD = [];                                                            % return empty PD
else
    
    % Read the variable values from the parameters table:
    
    [nR,~]=size(parameters);
    for k=1:nR 
        temp=parameters{(parameters.Row{k}),'Value'};                   % take the value for each variable
        if ischar(temp{:})                                              % convert from char to number array if needed
            temp2=str2num(temp{:});
            temp={temp2};
        end
        obj.(parameters.Row{k})=temp{:};                                % create the obj. structure based on the name of each variable
        temp=[];
    end
    
    PD=MakeStruct(obj);                                             % call the function building the structure's voxels cloud

end
end

function PD = MakeStruct(obj)

% Variables defined in the main function can be used as obj.var1(1), 
% obj.var1(2), obj.var2, obj.pr... to build the voxel list in the printing order.
% If intermediate parts needs to be build, you can use subfunction and
% assemble the parts before the output PD.

    y = subfunction1(obj);
    v = subfunction2(obj);

%%% Output print data definition %%% %%%%%%% 
% of x,y,z coordinates and pressure pr of the voxel list. 
% The format required is n*1 for each x,y,z and pr fields, 
% where n is the number of voxels (column arrays, 1 line per voxel)

    PD.x = obj.var1(:); 
    PD.y = [y;v];
    PD.z = [0.5;10];
    PD.pr = obj.pr*ones(length(PD.z),1);
    
end


%%%% user-defined subfunctions %%%%

function output=subfunction1(obj)
    output=obj.var2;
end

function output=subfunction2(obj)
    output= obj.var1(2);
end