function PD = Helix(parameters, CreateDefaultFlag, app)

if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag
    
    
    Variable    ={'n';'Hi';'hf';'Ri';'rf';'F';'p';'radius_smothing';'start';'finish';'support';'support_spacing';'z_gap';'pr'};
    Description ={'Number of turns';'Initial height';'Vertical separation';'Initial radius';'Radial separation';'Phase';'Voxel distance';'Radial smoothing';'Starting pillar flag';'Final pillar flag';'Support pillars flag';'Supports spacing';'Pillars Z gap';'Pressure'};
    Value       ={5; 0; 20; 20; 0; 0; 0.5; 0; 1; 0; 0; 300; 2; 25};
    Unit        ={'turns';'um';'um';'um';'um';'deg';'um';'logical';'logical';'logical';'logical';'um';'um';'mbar'};
    
    
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);
    save([app.parameters_path], 'ParametersTable');
    PD = []; % return empty PD
else
    % Get the variable values from the parameters:
    
    [nR,nC]=size(parameters);
    for k=1:nR
        temp=parameters{(parameters.Row{k}),'Value'};
        obj.(parameters.Row{k})=temp{:};
        temp=[];
    end

    PD=MakeHelix(obj);
end
end


function PD=MakeHelix(obj)

    obj.Hf=obj.n*obj.hf+obj.Hi;
    obj.Rf=obj.n*obj.rf+obj.Ri;
    obj.f=obj.F/360*2*pi;
    S=[];
    s=0;
    
    while s<=1
        S=[S;s];
        s=s+ds(obj,s);
    end
    
    Points=[];
    
    if obj.start ~= 0
        Points=[Points;add_pillar(obj,0)];
    end
    
    if obj.finish ~= 0
        Points=[Points;add_pillar(obj,1)];
    end
    
    if obj.support ~= 0
        Points=[Points;add_support(obj,S)];
        if obj.support ~= 2
            Points=[Points;V(obj,S(1:end))];
        end
    else
        Points=[Points;V(obj,S(1:end))];
    end
    
    
    PD.x=Points(:,1);
    PD.y=Points(:,2);
    PD.z=Points(:,3);
    PD.pr = obj.pr*ones(length(PD.z),1);
    
    % removes voxels too close from substrate
PD.x(PD.z<obj.p)=[];
PD.y(PD.z<obj.p)=[];
PD.pr(PD.z<obj.p)=[];
PD.z(PD.z<obj.p)=[];

end

function x=add_pillar(opt,S)
X=V(opt,S);
z=[opt.p:opt.p:X(3)-opt.z_gap]';
l=ones(length(z),1);
x=[l*X(1),l*X(2),z];
end



function x=add_support(opt,S)
x=[];
n=round(opt.support_spacing/opt.p);
nTot=length(S)-1;
for i=1+n:n:nTot
    X=V(opt,S(i));
    z=[opt.p:opt.p:X(3)-opt.z_gap]';
    l=ones(length(z),1);
    x=[x;l*X(1),l*X(2),z];
end
end

function x=r(opt,s)
if opt.radius_smothing == 0
    x=opt.Ri+(opt.Rf-opt.Ri).*s;
else
    x=-(opt.Rf-opt.Ri)*0.5*cos(s*pi)+(opt.Rf+opt.Ri)*0.5;
end
end

function x=h(opt,s)
x=opt.Hi+(opt.Hf-opt.Hi).*s;
end

function x=w(opt,s)
x=opt.f+2*pi*(opt.n).*s;
end

function x=V(opt,s)
x=[r(opt,s).*cos(w(opt,s)) r(opt,s).*sin(w(opt,s)) h(opt,s)];
end

function x=ds(opt,s)
po=V(opt,s);
pf=V(opt,s+0.000001);
x=opt.p*0.000001/norm(po-pf);
end