function PD = Ladder(parameters, CreateDefaultFlag,app)

if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag

    Variable    ={'d';'Id';'n';'h';'w';'pr'};
    Description ={'Voxel distance';'Intersection distance';'number of steps';'Step height';'Step width';'Pressure'};
    Value       ={0.5;2;3;20;30;60};
    Unit        ={'um';'um';'steps';'um';'um';'mbar'};
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);
    save([app.parameters_path], 'ParametersTable');
    PD = []; % return empty PD
else
    % Get the variable values from the parameters:
    
    [nR,nC]=size(parameters);
    for k=1:nR
        temp=parameters{(parameters.Row{k}),'Value'};
        obj.(parameters.Row{k})=temp{:};
        temp=[];
    end
    
    PD=MakeLadder(obj);
end
end

function PD=MakeLadder(obj)
obj.X=[];
obj.Y=[];
obj.Z=[];

n1=round((obj.h-obj.Id)/obj.d)+1;
z1=linspace(obj.d,obj.h-obj.Id,n1)';
y1=zeros(n1,1);
x1=ones(n1,1)*obj.w;

n2=round((obj.h-obj.d)/obj.d)+1;
z2=linspace(obj.d,obj.h-obj.d,n2)';
y2=zeros(n2,1);
x2=y2;

n3=round(obj.w/obj.d)+1;
x3=linspace(0,obj.w,n3)';
y3=zeros(n3,1);
z3=ones(n3,1)*obj.h;

X=[x1;x2;x3];
Y=[y1;y2;y3];
Z=[z1;z2;z3];

for i=1:obj.n
    obj.X=[obj.X;X];
    obj.Y=[obj.Y;Y];
    obj.Z=[obj.Z;Z+obj.h*(i-1)];
end
    
    PD.x=obj.X;
    PD.y=obj.Y;
    PD.z=obj.Z;
    PD.pr = obj.pr*ones(length(PD.z),1);
end