function PD = Plane(parameters, CreateDefaultFlag, app)

if nargin < 2
    CreateDefaultFlag = 0;
end

if CreateDefaultFlag

    Variable    ={'X';'Y';'zo';'Z_gap';'dv';'dl';'beams';'pr'};                 % name of each variable used for this scruture
    Description ={'Size X';'Size Y';'Plane Z level';'Z Gap';'voxel pitch';'strand pitch';'BeamsFlag';'Pressure'};    % description of each variable
    Value       ={60;60;5;3;0.5;1;1;30};                                  % default values 
    Unit        ={'um';'um';'um';'um';'um';'um';'logical';'mbar'};                % units of each variable
    
    Variable{end+1}='script'; Description{end+1}='Script'; Value{end+1}={mfilename}; Unit{end+1}='-'; %add script name to parameters for script memory management in the VCG
    ParametersTable=table(Value,Description,Unit,'RowNames',Variable);  % build the default parameters table
    save([app.parameters_path], 'ParametersTable');                     % save the default parameters table to disk    
    PD = [];                                                            % return empty PD
else
    
    % Get the variable values from the parameters table:
    
    [nR,nC]=size(parameters);
    for k=1:nR 
        temp=parameters{(parameters.Row{k}),'Value'};              % take the value for each variable
        if ischar(temp{:})                                         % convert from char to number array if needed
            temp2=str2num(temp{:});
            temp={temp2};
        end
        obj.(parameters.Row{k})=temp{:};                           % create the obj. structure based on the name of each variable
        temp=[];
    end
    
    PD=MakePlane(obj);                                             % call the function building the structure's voxels cloud

end
end

function PD = MakePlane(obj)

obj.xo=-obj.X*0.5;
obj.yo=-obj.Y*0.5;
obj.Dz=obj.zo-obj.Z_gap;

if obj.Dz<=0
    obj.beams=0;
end

if obj.beams %remove beams if BeamFlag set to 0
    Points=beam(obj);
else
    Points=[];
end

Points=[Points;plane(obj)];

PD.x=Points(:,1);
PD.y=Points(:,2);
PD.z=Points(:,3);
PD.pr = obj.pr*ones(length(PD.z),1);
% removes voxels too close from substrate
% PD.x(PD.z<obj.dv)=[];
% PD.y(PD.z<obj.dv)=[];
% PD.pr(PD.z<obj.dv)=[];
% PD.z(PD.z<obj.dv)=[];
end

function x=beam(obj)
    V=[obj.xo,obj.yo,obj.Dz];
    n=round(norm(V)/obj.dv);    
    V=[linspace(0,obj.xo,n)' linspace(0,obj.yo,n)' linspace(0,obj.Dz,n)'];
    V=V(2:end,:);
    x=ones(n-1*4,3);
    for i = 1:n-1
        x((i-1)*4+1:i*4,:)=[V(i,:);V(i,:).*[-1 1 1];V(i,:).*[-1 -1 1];V(i,:).*[1 -1 1]];        
    end
end

function x=plane(obj)
    x=[];
    n_x=round(obj.X/obj.dv);
    n_y=round(obj.Y/obj.dl);
    h_odd=linspace(obj.xo,obj.xo+obj.X,n_x)';
    dh=(h_odd(2)-h_odd(1))/2;
    h_even=linspace(obj.xo+obj.X-dh,obj.xo+dh,n_x-1)';
    vert=linspace(obj.yo,obj.yo+obj.Y,n_y)';
    for i=1:length(vert)
        if mod(i,2) == 0
            vx=h_even;
            n=n_x-1;
        else
            vx=h_odd;
            n=n_x;
        end
        x=[x;vx,ones(n,1)*vert(i)];        
    end
    n=length(x(:,1));
    x=[x,ones(n,1)*obj.zo];
end